syntax = "proto3";

import "google/protobuf/timestamp.proto";
import "TimestampValue.proto";

option java_package = "io.confluent.kafka.serializers.protobuf.test";

// Wrapper message for `Timestamp`.
message KeyTimestampValue {
  int32 key = 1;
  TimestampValue value = 2;
  google.protobuf.Timestamp value2 = 3;
}
